/* tworuns.c
   March 2003, Ziheng Yang

   This program compares output from two separate ML runs analyzying
   multiple data sets to assemble a result file.  Because of multiple peaks 
   or convergence problems, two runs for the same analysis might not generate 
   the same results.  Then we should use the results corresponding to the 
   higher ln likelihood.  This program takes two input files which have 
   summary results from two runs, one line for each data set.  The program
   takes one line from each of the two input files and compare the
   first field, which is a kind of lnLcolumn and should be identical between the 
   two input files, and an lnL column.  The program decides which run generated 
   a higher lnL, and copy the line from that run into the output file.

   This is useful when you analyze the same set of simulated replicate
   data sets twice, using different starting values.  You write one
   line of output from each data set.  This program then compares the
   lnL values between the two runs and collect the result line from
   the better run into a file.
   The program assumes that all fields are numerical numbers and text is not 
   allowed.

     cl -O2 -Ot -W4 tworuns.c tools.c
     cc -o tworuns tworuns.c tools.c -lm

     tworuns <rstfile1> <rstfile2> <lnLColumn> 
*/

#include "paml.h"
#define MAXNFIELDS  1000
#define MAXLLINE    64000

int main(int argc, char* argv[])
{
   FILE *fout, *fin[2];
//   char infile[2][96]={"\papers\karl\MLEs.3s.2w.1", "\papers\karl\rst1"};
   char infile[2][96]={"f1", "f2"}, outfile[96]="out.txt";
   int index=0, lnLcolumn=13, i, nrecords=0, lline=MAXLLINE, nfields[2],fields[MAXNFIELDS];
   int chosen, nfileread;
   char line[2][MAXLLINE];
   double f[2], indexfield[2];

   puts("Result goes to out.txt\n\tUsage: \n\ttworuns <file1> <file2> <lnLcolumn>\n");
   if(argc>1) strcpy(infile[0], argv[1]);
   if(argc>2) strcpy(infile[1], argv[2]);
   if(argc>3) sscanf(argv[3], "%d", &lnLcolumn);
   lnLcolumn--;

   fin[0]=gfopen(infile[0],"r");
   fin[1]=gfopen(infile[1],"r");
   fout=(FILE*)gfopen(outfile,"w");

   for(nrecords=0; ; nrecords++) {
      for(i=0,nfileread=0; i<2; i++) { 
         nfields[i]=0; f[i]=0; line[i][0]='\0';
         if(!fgets(line[i], lline, fin[i])) continue;
         nfileread++; 
         nfields[i]=splitline(line[i],fields);
         indexfield[i]=fields[index];

         /* if(nfields[i]) sscanf(line[i], "%lf", &indexfield[i]); */
         if(nfields[i]>lnLcolumn) sscanf(line[i]+fields[lnLcolumn], "%lf", &f[i]);
      }
      if(nfileread==0 || (nfields[0]==0 && nfields[1]==0)) break;
      if(nfileread==2 && indexfield[0]!=indexfield[1]) puts("first field not identical?");
      chosen=(nfields[0]<nfields[1]);
      if(nfields[0]==nfields[1]) chosen=(f[0]<f[1]);

      fprintf(fout, "%s", line[chosen]);
      printf("record %5d: fields: %3d %3d  %10.3f - %10.3f = %10.3f (%d)\n", 
         nrecords+1, nfields[0], nfields[1],f[0],f[1],f[0]-f[1],chosen);
   }
}
